<?php
// $Id: database.php,v 1.12 2003/11/03 18:55:27 rcastley Exp $
/**
* Content code
* @package Mambo Open Source
* @Copyright (C) 2000 - 2003 Miro International Pty Ltd
* @ All rights reserved
* @ Mambo Open Source is Free Software
* @ Released under GNU/GPL License : http://www.gnu.org/copyleft/gpl.html
* @version $Revision: 1.12 $
**/

defined( '_VALID_MOS' ) or die( 'Direct Access to this location is not allowed.' );

/**
* @package MOS
* @copyright 2000-2003 Miro International Pty Ltd
* @license http://www.gnu.org/copyleft/gpl.html. GNU Public License
* @version 2.5 $Revision: 1.12 $
* @author Andrew Eddie <eddieajau@user.sourceforge.net>
*/

/**
* Database connector class
*
* <b>Example:</b>
* <code>
* $db = new database( 'localhost', 'root', '', 'mambo', 'mos_' );
* $db->setQuery( "SELECT * FROM #__users" );
* if ($db->query()) {
*	echo 'ok';
* } else {
*	echo $db->stderr();
* }
* </code>
*
* @package MOS
* @subpackage LoudMouth
* @author Andrew Eddie <eddieajau@user.sourceforge.net>
*/
class database {
/** @var string Internal variable to hold the query sql */
	var $_sql='';
/** @var int Internal variable to hold the database error number */
	var $_errorNum=0;
/** @var string Internal variable to hold the database error message */
	var $_errorMsg='';
/** @var string Internal variable to hold the prefix used on all database tables */
	var $_table_prefix='';
/** @var Internal variable to hold the connector resource */
	var $_resource='';
/** @var Internal variable to hold the last query cursor */
	var $_cursor=null;

/**
* Database object constructor
* @param string Database host
* @param string Database user name
* @param string Database user password
* @param string Database name
* @param string Common prefix for all tables
*/
	function database( $host='localhost', $user, $pass, $db, $table_prefix ) {
	// perform a number of fatality checks, then die gracefully
		function_exists( 'mysql_connect' )
			or die( 'FATAL ERROR: MySQL support not available.  Please check your configuration.' );
		$this->_resource = mysql_connect( $host, $user, $pass )
			or die( 'FATAL ERROR: Connection to database server failed.' );
		mysql_select_db($db)
			or die( "FATAL ERROR: Database not found. Operation failed with error: ".mysql_error());
		$this->_table_prefix = $table_prefix;
	}
/**
* Execute a database query and returns the result
* @param string The SQL query
* @return resource Database resource identifier.  Refer to the PHP manual for more information.
* @deprecated This function is included for tempoary backward compatibility
*/
	function openConnectionWithReturn($query){
		$result=mysql_query($query) or die("Query failed with error: ".mysql_error());
		return $result;
	}
/**
* Execute a database query
* @param string The SQL query
* @deprecated This function is included for temporary backward compatibility
*/
	function openConnectionNoReturn($query){
		mysql_query($query) or die("Query failed with error: ".mysql_error());
	}
/**
* @return int The error number for the most recent query
*/
	function getErrorNum() {
		return $this->_errorNum;
	}
/**
* @return string The error message for the most recent query
*/
	function getErrorMsg() {
		return str_replace( array( "\n", "'" ), array( '\n', "\'" ), $this->_errorMsg );
	}
/**
* Get a database escaped string
* @return string
*/
	function getEscaped( $text ) {
		return mysql_escape_string( $text );
	}
/**
* Sets the SQL query string for later execution.
*
* This function replaces a string identifier <var>$prefix</var> with the
* string held is the <var>_table_prefix</var> class variable.
*
* @param string The SQL query
* @param string The common table prefix
*/
	function setQuery( $sql, $prefix='#__' ) {
		$this->_sql = str_replace( $prefix, $this->_table_prefix, $sql );
	}
/**
* @return string The current value of the internal SQL vairable
*/
	function getQuery() {
		return "<pre>$this->_sql</pre>";
	}
/**
* Execute the query
* @return mixed A database resource if successful, FALSE if not.
*/
	function query() {
		$this->_errorNum = 0;
		$this->_errorMsg = '';
		$this->_cursor = mysql_query( $this->_sql, $this->_resource );
		if( !$this->_cursor ) {
			$this->_errorNum = mysql_errno( $this->_resource );
			$this->_errorMsg = mysql_error( $this->_resource )." SQL=$this->_sql";
			return false;
		}
		return $this->_cursor;
	}
/**
* Diagnostic function
*/
	function explain() {
		$temp = $this->_sql;
		$this->_sql = "EXPLAIN $this->_sql";
		$this->query();

		if (!($cur = $this->query())) {
			return null;
		}
		$first = true;

		$buf = "<table cellspacing=\"1\" cellpadding=\"2\" border=\"0\" bgcolor=\"#000000\" align=\"center\">";
		$buf .= $this->getQuery();
		while ($row = mysql_fetch_assoc( $cur )) {
			if ($first) {
				$buf .= "<tr>";
				foreach ($row as $k=>$v) {
					$buf .= "<th bgcolor=\"#ffffff\">$k</th>";
				}
				$buf .= "</tr>";
				$first = false;
			}
			$buf .= "<tr>";
			foreach ($row as $k=>$v) {
				$buf .= "<td bgcolor=\"#ffffff\">$v</td>";
			}
			$buf .= "</tr>";
		}
			$buf .= "</table><br />&nbsp;";
		mysql_free_result( $cur );

		$this->_sql = $temp;

		return "<div style=\"background-color:#FFFFCC\" align=\"left\">$buf</div>";
	}
/**
* @return int The number of rows returned from the most recent query.
*/
	function getNumRows( $cur=null ) {
		return mysql_num_rows( $cur ? $cur : $this->_cursor );
	}

/**
* This method loads the first field of the first row returned by the query.
*
* @return The value returned in the query or null if the query failed.
*/
	function loadResult() {
		if (!($cur = $this->query())) {
			return null;
		}
		$ret = null;
		if ($row = mysql_fetch_row( $cur )) {
			$ret = $row[0];
		}
		mysql_free_result( $cur );
		return $ret;
	}
/**
* Load an array of single field results into an array
*/
	function loadResultArray($numinarray = 0) {
		if (!($cur = $this->query())) {
			return null;
		}
		$array = array();
		while ($row = mysql_fetch_row( $cur )) {
			$array[] = $row[$numinarray];
		}
		mysql_free_result( $cur );
		return $array;
	}
/**
* This global function loads the first row of a query into an object
*
* If an object is passed to this function, the returned row is bound to the existing elements of <var>object</var>.
* If <var>object</var> has a value of null, then all of the returned query fields returned in the object.
* @param string The SQL query
* @param object The address of variable
*/
	function loadObject( &$object ) {
		if ($object != null) {
			if (!($cur = $this->query())) {
				return false;
			}
			if ($array = mysql_fetch_assoc( $cur )) {
				mysql_free_result( $cur );
				mosBindArrayToObject( $array, $object );
				return true;
			} else {
				return false;
			}
		} else {
			if ($cur = $this->query()) {
				if ($object = mysql_fetch_object( $cur )) {
					mysql_free_result( $cur );
					return true;
				} else {
					$object = null;
					return false;
				}
			} else {
				return false;
			}
		}
	}
/**
* Load a list of database objects
* @param string The field name of a primary key
* @return array If <var>key</var> is empty as sequential list of returned records.
* If <var>key</var> is not empty then the returned array is indexed by the value
* the database key.  Returns <var>null</var> if the query fails.
*/
	function loadObjectList( $key='' ) {
		if (!($cur = $this->query())) {
			return null;
		}
		$array = array();
		while ($row = mysql_fetch_object( $cur )) {
			if ($key) {
				$array[$row->$key] = $row;
			} else {
				$array[] = $row;
			}
		}
		mysql_free_result( $cur );
		return $array;
	}
/**
* Document::db_insertObject()
*
* { Description }
*
* @param [type] $keyName
* @param [type] $verbose
*/
	function insertObject( $table, &$object, $keyName = NULL, $verbose=false ) {
		$fmtsql = "INSERT INTO $table ( %s ) VALUES ( %s ) ";
		foreach (get_object_vars( $object ) as $k => $v) {
			if (is_array($v) or is_object($v) or $v == NULL) {
				continue;
			}
			if ($k[0] == '_') { // internal field
				continue;
			}
			$fields[] = "`$k`";
			$values[] = "'" . $this->getEscaped( $v ) . "'";
		}
		$this->setQuery( sprintf( $fmtsql, implode( ",", $fields ) ,  implode( ",", $values ) ) );
		($verbose) && print "$sql<br />\n";
		if (!$this->query()) {
			return false;
		}
		$id = mysql_insert_id();
		($verbose) && print "id=[$id]<br />\n";
		if ($keyName && $id) {
			$object->$keyName = $id;
		}
		return true;
	}

/**
* Document::db_updateObject()
*
* { Description }
*
* @param [type] $updateNulls
*/
	function updateObject( $table, &$object, $keyName, $updateNulls=true ) {
		$fmtsql = "UPDATE $table SET %s WHERE %s";
		foreach (get_object_vars( $object ) as $k => $v) {
			if( is_array($v) or is_object($v) or $k[0] == '_' ) { // internal or NA field
				continue;
			}
			if( $k == $keyName ) { // PK not to be updated
				$where = "$keyName='" . $this->getEscaped( $v ) . "'";
				continue;
			}
			if ($v === NULL && !$updateNulls) {
				continue;
			}
			if( $v == '' ) {
				$val = "''";
			} else {
				$val = "'" . $this->getEscaped( $v ) . "'";
			}
			$tmp[] = "`$k`=$val";
		}
		$this->setQuery( sprintf( $fmtsql, implode( ",", $tmp ) , $where ) );
		return $this->query();
	}

	function getObjectList( $index=null, $maxrows=NULL ) {
		$this->_errorNum = 0;
		$this->_errorMsg = '';

		if (!($cur = mysql_query( $this->_sql ))) {;
			$this->_errorNum = mysql_errno();
			$this->_errorMsg = mysql_error();
			return false;
		}
		$list = array();
		$cnt = 0;
		while ($obj = mysql_fetch_object( $cur )) {
			if ($index) {
				$list[$obj->$index] = $obj;
			} else {
				$list[] = $obj;
			}
			if( $maxrows && $maxrows == $cnt++ ) {
				break;
			}
		}
		mysql_free_result( $cur );
		return $list;
	}
/**
* @param boolean If TRUE, displays the last SQL statement sent to the database
* @return string A standised error message
*/
	function stderr( $showSQL = false ) {
		return "DB function failed with error number $this->_errorNum"
			."<br /><font color=\"red\">$this->_errorMsg</font>"
			.($showSQL ? "<br />SQL = <pre>$this->_sql</pre>" : '');
	}

	function insertid()
	{
		return mysql_insert_id();
	}

	function getVersion()
	{
		return mysql_get_server_info();
	}

}

/**
* mosDBTable Abstract Class.
*
* Parent classes to all database derived objects.  Customisation will generally
* not involve tampering with this object.
* @package MOS
* @author Andrew Eddie <eddieajau@users.sourceforge.net
* @abstract
*/
class mosDBTable {
/** @var string Name of the table in the db schema relating to child class */
	var $_tbl = '';
/** @var string Name of the primary key field in the table */
	var $_tbl_key = '';
/** @var string Error message */
	var $_error = '';
/** @var mosDatabase Database connector */
	var $_db = null;

/**
 *	Object constructor to set table and key field
 *
 *	Can be overloaded/supplemented by the child class
 *	@param string $table name of the table in the db schema relating to child class
 *	@param string $key name of the primary key field in the table
 */
	function mosDBTable( $table, $key, &$db ) {
		$this->_tbl = $table;
		$this->_tbl_key = $key;
		$this->_db = $db;
	}
/**
 *	@return string Returns the error message
 */
	function getError() {
		return $this->_error;
	}
/**
 * Gets the value of the class variable
 * @param string The name of the class variable
 * @return mixed The value of the class var (or null if no var of that name exists)
 */
	function get( $_property ) {
		if(isset( $this->$_property )) {
			return $this->$_property;
		} else {
			return null;
		}
	}
/**
 * Set the value of the class variable
 * @param string The name of the class variable
 * @param mixed The value to assign to the variable
 */
	function set( $_property, $_value ) {
		$this->$_property = $_value;
	}
/**
 *	binds a named array/hash to this object
 *
 *	can be overloaded/supplemented by the child class
 *	@param array $hash named array
 *	@return null|string	null is operation was satisfactory, otherwise returns an error
 */
	function bind( $array ) {
		if (!is_array( $array )) {
			$this->_error = get_class( $this )."::bind failed.";
			return false;
		} else {
			return mosBindArrayToObject( $array, $this );
		}
	}

/**
 *	binds an array/hash to this object
 *	@param int $oid optional argument, if not specifed then the value of current key is used
 *	@return any result from the database operation
 */
	function load( $oid=null ) {
		$k = $this->_tbl_key;
		if ($oid !== null) {
			$this->$k = $oid;
		}
		$oid = $this->$k;
		if ($oid === null) {
			return false;
		}
		$this->_db->setQuery( "SELECT * FROM $this->_tbl WHERE $this->_tbl_key='$oid'" );
		return $this->_db->loadObject( $this );
	}

/**
 *	generic check method
 *
 *	can be overloaded/supplemented by the child class
 *	@return boolean True if the object is ok
 */
	function check() {
		return true;
	}

/**
* Inserts a new row if id is zero or updates an existing row in the database table
*
* Can be overloaded/supplemented by the child class
* @param boolean If false, null object variables are not updated
* @return null|string null if successful otherwise returns and error message
*/
	function store( $updateNulls=false ) {
		$k = $this->_tbl_key;
		if( $this->$k ) {
			$ret = $this->_db->updateObject( $this->_tbl, $this, $this->_tbl_key, $updateNulls );
		} else {
			$ret = $this->_db->insertObject( $this->_tbl, $this, $this->_tbl_key );
		}
		if( !$ret ) {
			$this->_error = get_class( $this )."::store failed <br />" . $this->_db->getErrorMsg();
			return false;
		} else {
			return true;
		}
	}
/**
*/
	function move( $dirn, $where='' ) {
		$k = $this->_tbl_key;

		$sql = "SELECT $this->_tbl_key, ordering FROM $this->_tbl";

		if ($dirn < 0) {
			$sql .= "\nWHERE ordering < $this->ordering";
			$sql .= ($where ? "\n	AND $where" : '');
			$sql .= "\nORDER BY ordering DESC\nLIMIT 1";
		} else if ($dirn > 0) {
			$sql .= "\nWHERE ordering > $this->ordering";
			$sql .= ($where ? "\n	AND $where" : '');
			$sql .= "\nORDER BY ordering\nLIMIT 1";
		} else {
			$sql .= "\nWHERE ordering = $this->ordering";
			$sql .= ($where ? "\n	AND $where" : '');
			$sql .= "\nORDER BY ordering\nLIMIT 1";
		}

		$this->_db->setQuery( $sql );

		$row = null;
		if ($this->_db->loadObject( $row )) {
			print_r($row);
			$this->_db->setQuery( "UPDATE $this->_tbl SET ordering='$row->ordering'"
				. "\nWHERE $this->_tbl_key='".$this->$k."'"
			);

			$this->_db->query();

			$this->_db->setQuery( "UPDATE $this->_tbl SET ordering='$this->ordering'"
				. "\nWHERE $this->_tbl_key='".$row->$k."'"
			);

			$this->_db->query();

			$this->ordering = $row->ordering;
		} else {
			$this->_db->setQuery( "UPDATE $this->_tbl SET ordering='$this->ordering'"
				. "\nWHERE $this->_tbl_key='".$this->$k."'"
			);

			$this->_db->query();
		}
	}
/**
* Compacts the ordering sequence of the selected records
* @param string Additional where query to limit ordering to a particular subset of records
*/
	function updateOrder( $where='' ) {
		if (!array_key_exists( 'ordering', get_class_vars( get_class( $this ) ) )) {
			$this->_error = "WARNING: ".get_class( $this )." does not support ordering.";
			return false;
		}
		$this->_db->setQuery( "SELECT $this->_tbl_key FROM $this->_tbl"
			. "\nWHERE id <> '$this->id'"
			. ($where ? "\n	AND $where" : '')
			. "\nORDER BY ordering"
		);
		if (!($orders = $this->_db->loadObjectList())) {
			$this->_error = $this->_db->getErrorMsg();
			return false;
		}
		$shift = 0;
		for ($i=0, $n=count( $orders ); $i < $n; $i++) {
			$order = $i + 1;
			$item = &$orders[$i];
			if ($order == $this->ordering && $shift == 0) {
				$shift = 1;
			}
			$order += $shift;
			$this->_db->setQuery( "UPDATE $this->_tbl"
				. "\nSET ordering='$order' WHERE id='$item->id'"
			);
			$this->_db->query();

		}
		// if we didn't reorder the current record, make it last
		if ($shift == 0) {
			$order = $i + 1 + $shift;
			$this->_db->setQuery( "UPDATE $this->_tbl"
				. "\nSET ordering='$order' WHERE id='$this->id'"
			);
			$this->_db->query();
		}
		return true;
	}
/**
 *	Generic check for whether dependancies exist for this object in the db schema
 *
 *	can be overloaded/supplemented by the child class
 *	@param string $msg Error message returned
 *	@param int Optional key index
 *	@param array Optional array to compiles standard joins: format [label=>'Label',name=>'table name',idfield=>'field',joinfield=>'field']
 *	@return true|false
 */
	function canDelete( $oid=null, $joins=null ) {
		$k = $this->_tbl_key;
		if ($oid) {
			$this->$k = intval( $oid );
		}
		if (is_array( $joins )) {
			$select = "$k";
			$join = "";
			foreach( $joins as $table ) {
				$select .= ",\nCOUNT(DISTINCT {$table['idfield']}) AS {$table['idfield']}";
				$join .= "\nLEFT JOIN {$table['name']} ON {$table['joinfield']} = $k";
			}
			$this->_db->setQuery( "SELECT $select\nFROM $this->_tbl\n$join\nWHERE $k = ".$this->$k." GROUP BY $k" );

			if ($obj = $this->_db->loadObject()) {
				$this->_error = $this->_db->getErrorMsg();
				return false;
			}
			$msg = array();
			foreach( $joins as $table ) {
				$k = $table['idfield'];
				if ($obj->$k) {
					$msg[] = $AppUI->_( $table['label'] );
				}
			}

			if (count( $msg )) {
				$this->_error = "noDeleteRecord" . ": " . implode( ', ', $msg );
				return false;
			} else {
				return true;
			}
		}

		return true;
	}

/**
 *	Default delete method
 *
 *	can be overloaded/supplemented by the child class
 *	@return true if successful otherwise returns and error message
 */
	function delete( $oid=null ) {
		//if (!$this->canDelete( $msg )) {
		//	return $msg;
		//}

		$k = $this->_tbl_key;
		if ($oid) {
			$this->$k = intval( $oid );
		}

		$this->_db->setQuery( "DELETE FROM $this->_tbl WHERE $this->_tbl_key = '".$this->$k."'" );

		if ($this->_db->query()) {
			return true;
		} else {
			$this->_error = $this->_db->getErrorMsg();
			return false;
		}
	}

	function checkout( $who, $oid=null ) {
		if (!array_key_exists( 'checked_out', get_class_vars( get_class( $this ) ) )) {
			$this->_error = "WARNING: ".get_class( $this )." does not support checkouts.";
			return false;
		}
		$k = $this->_tbl_key;
		if ($oid !== null) {
			$this->$k = $oid;
		}
		$time = date( "%Y-%m-%d H:i:s" );
		if (intval( $who )) {
			// new way of storing editor, by id
			$this->_db->setQuery( "UPDATE $this->_tbl"
				. "\nSET checked_out='$who', checked_out_time='$time'"
				. "\nWHERE $this->_tbl_key='".$this->$k."'"
				);
		} else {
			// old way of storing editor, by name
			$this->_db->setQuery( "UPDATE $this->_tbl"
				. "\nSET checked_out='1', checked_out_time='$time', editor='".$who."' "
				. "\nWHERE $this->_tbl_key='".$this->$k."'"
				);
		}
		return $this->_db->query();
	}

	function checkin( $oid=null ) {
		if (!array_key_exists( 'checked_out', get_class_vars( get_class( $this ) ) )) {
			$this->_error = "WARNING: ".get_class( $this )." does not support checkin.";
			return false;
		}
		$k = $this->_tbl_key;
		if ($oid !== null) {
			$this->$k = $oid;
		}
		$time = date("H:i:s");
		$this->_db->setQuery( "UPDATE $this->_tbl"
			. "\nSET checked_out='0', checked_out_time='0000-00-00 00:00:00'"
			. "\nWHERE $this->_tbl_key='".$this->$k."'"
		);
		return $this->_db->query();
	}

	function hit( $oid=null ) {
		global $mosConfig_enable_log_items;

		$k = $this->_tbl_key;
		if ($oid !== null) {
			$this->$k = intval( $oid );
		}
		$this->_db->setQuery( "UPDATE $this->_tbl SET hits=(hits+1) WHERE $this->_tbl_key='$this->id'" );
		$this->_db->query();
		
		if (@$mosConfig_enable_log_items) {
			$now = date( "Y-m-d" );
			$this->_db->setQuery( "SELECT hits"
				. "\nFROM #__core_log_items"
				. "\nWHERE time_stamp='$now' AND item_table='$this->_tbl' AND item_id='".$this->$k."'"
			);
			$hits = intval( $this->_db->loadResult() );
			if ($hits) {
				$this->_db->setQuery( "UPDATE #__core_log_items SET hits=(hits+1)"
					. "\nWHERE time_stamp='$now' AND item_table='$this->_tbl' AND item_id='".$this->$k."'"
				);
				$this->_db->query();
			} else {
				$this->_db->setQuery( "INSERT INTO #__core_log_items VALUES"
					. "\n('$now','$this->_tbl','".$this->$k."','1')"
				);
				$this->_db->query();
			}
		}
	}

/**
* Generic save function
* @param array Source array for binding to class vars
* @param string Filter for the order updating
* @returns TRUE if completely successful, FALSE if partially or not succesful.
*/
	function save( $source, $order_filter ) {
		if (!$this->bind( $_POST )) {
			return false;
		}
		if (!$this->check()) {
			return false;
		}
		if (!$this->store()) {
			return false;
		}
		if (!$this->checkin()) {
			return false;
		}
		$filter_value = $this->$order_filter;
		$this->updateOrder( $order_filter ? "`$order_filter`='$filter_value'" : "" );
		$this->_error = '';
		return true;
	}

/**
* Generic Publish/Unpublish function
* @param array An array of id numbers
* @param integer 0 if unpublishing, 1 if publishing
* @param integer The id of the user performnig the operation
*/
	function publish_array( $cid=null, $publish=1, $myid=0 ) {
		if (!is_array( $cid ) || count( $cid ) < 1) {
			$this->_error = "No items selected.";
			return false;
		}

		$cids = implode( ',', $cid );

		$this->_db->setQuery( "UPDATE $this->_tbl SET published='$publish'"
		. "\nWHERE id IN ($cids) AND (checked_out=0 OR (checked_out='$myid'))"
		);
		if (!$this->_db->query()) {
			$this->_error = $this->_db->getErrorMsg();
			return false;
		}

		if (count( $cid ) == 1) {
			$this->checkin( $cid[0] );
		}
		$this->_error = '';
		return true;
	}
}
?>
